/**
 * ============================================================
 * Perceptron.java: Simple perceptron model ....
 *
 * @author Orhan Demirel
 * ============================================================
 */

package neural;

import java.util.Random;

public class Perceptron {
    private double[] weights;
    private double threshold    = 0.2;
    private double learningrate = 0.1;
    private int epoch = 100;
    private int noInputs;
    private int noOutputs;

    // Constructor ....

    public Perceptron() {}

    // Initialize model parameters ...

    public void setThreshold( double threshhold ) {
       this.threshold = threshold;
    }

    public void setLearningRate( double learningrate ) {
       this.learningrate = learningrate;
    }

    public void setEpochs( int epoch ) {
       this.epoch = epoch;
    }

    // ====================================================
    // Train perception model ...
    // ====================================================

    public void trainModel(double[][] inputs, int[] outputs ) {

        System.out.printf("--- Enter trainModel() ... \n");

        int n = inputs[0].length;
        int p = outputs.length;
        weights  = new double[n];
        noInputs  = n;
        noOutputs = 1;
        Random r = new Random();
        
        // Initialize weights ...

        System.out.printf("---   Initialize weights ... \n");

        for(int i=0;i<n;i++) {
            weights[i] = r.nextDouble();
        }

        // Main loop for training ...

        System.out.printf("---   Simple loop for training ... \n");

        for(int i=0; i < epoch; i++) {

            int totalError = 0;
            for(int j =0; j < p; j++) {
                int output = Output(inputs[j]);
                int error  = outputs[j] - output;
                
                totalError = totalError + error;
                for(int k = 0;k<n;k++) {
                    double delta = learningrate * inputs[j][k] * error;
                    weights[k] += delta;
                }
            }

            System.out.printf("---   epoch %3d: error = %d ... \n", i, totalError );

            if(totalError == 0)
                break;
        }

        System.out.printf("--- Leave trainModel() ... \n");
    }

    public int Output( double[] input ) {
        double sum = 0.0;
        for(int i=0;i<input.length;i++) {
           sum += weights[i]*input[i];
        }

        if( sum > threshold)
            return 1;
        else
            return 0;
    }

   // =========================================================
   // Create string();
   // =========================================================

   public String toString() {
      StringBuffer buf = new StringBuffer();

      buf.append("\n");
      buf.append("Perceptron \n");
      buf.append("===================================== \n");
      buf.append( String.format("--- threshold = %8.4f ...\n", this.threshold ) );
      buf.append( String.format("--- learning rate = %8.4f ...\n", this.learningrate ) );
      buf.append( String.format("--- max no epochs = %8d ...\n", this.epoch ) );

      // Add inputs / outputs ...

      buf.append( String.format("--- no inputs  = %4d ...\n", noInputs ) );
      buf.append( String.format("--- no outputs = %4d ...\n", noOutputs ) );

      // Add matrix of weight ...

      if( weights != null ) {
         buf.append( String.format("Weights: ...\n"));
         for(int i=0;i < weights.length; i++ ) {
            buf.append( String.format("--- weights[%2d] = %8.4f ...\n", i, weights[i] ));
         }
      }

      buf.append("===================================== \n");

      return buf.toString();
   }
}
