/**
  * ========================================================================
  * Cell.java: Object representation of a Cell in the Spreadsheet
  * ========================================================================
  */

package spreadsheet;

import java.util.*;

public class Cell {
   private String expression;    // expression in cell
   private Set<String> children; // list of cells which reference this
   private Set<String> parent;   // list of cells this references

   // Value of displayed cell ...

   private Object value; 

   // Flag to indicate #LOOP or otherwise
   // default: false - computeValue()

   private boolean isDirty = false;

   // Class constructor

   public Cell() {
      children = new TreeSet<String>();
      parent   = new TreeSet<String>();
   }

   /**
     * ==================================================================
     * Various accessors and mutators
     * ==================================================================
     */

   public String getExpression() {
      return expression;
   }

   public void setExpression(String expression) {
      this.expression = expression;
   }

   public Object getValue() {
      return value;
   }

   public void setValue(Object value) {
      this.value = value;
   }

   public Set<String> getChildren() {
      return children;
   }

   public Set<String> getParent() {
      return parent;
   }

   public void setParent(Set<String> parent) {
      this.parent = parent;
   }

   public boolean isDirty() {
		return isDirty;
   }

   public void setDirty(boolean isDirty) {
      this.isDirty = isDirty;
   }

   public void addChildren(String child) {
      children.add(child);
   }

   public void removeChildren(String child) {
      children.remove(child);
   }
}
